/*******************************************************************************
 * Quick access to the Components object.
 ******************************************************************************/
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
/*******************************************************************************
 * Component definitions.
 ******************************************************************************/
const FSS_CLASSNAME = "Netscape Friends' Activity Service";
const FSS_CLASSID = Components.ID("{bd2b371d-6b09-2640-9040-7335a5f36003}");
const FSS_CONTRACTID = "@netscape.com/fas/service;1";
/*******************************************************************************
 * Contract IDs.
 ******************************************************************************/
const PREF_CONTRACTID = "@mozilla.org/preferences-service;1";
const CAT_CONTRACTID = "@mozilla.org/categorymanager;1";
const OB_CONTRACTID = "@mozilla.org/observer-service;1";
const REQUEST_CONTRACTID = "@mozilla.org/xmlextras/xmlhttprequest;1";
const TIMER_CONTRACTID = "@mozilla.org/timer;1";
const BUNDLE_CONTRACTID = "@mozilla.org/intl/stringbundle;1";
const ALERT_CONTRACTID = "@mozilla.org/alerts-service;1";
const MEDIATOR_CONTRACTID = "@mozilla.org/appshell/window-mediator;1";
/*******************************************************************************
 * Observer topics
 ******************************************************************************/
const TOPIC_TIMER = "timer-callback";
const TOPIC_FAS = "netscape-fas";
const TOPIC_SHUTDOWN_APP = "quit-application";
const TOPIC_SHUTDOWN_XPCOM = "xpcom-shutdown";
const TOPIC_STARTUP_APP = "app-startup";
const TOPIC_STARTUP_UI = "final-ui-startup";
const TOPIC_PREF = "nsPref:changed";
const TOPIC_REQUEST = "http-on-examine-response";
const TOPIC_ALERT = "alertclickcallback";
/*******************************************************************************
 * Default values used across components.
 ******************************************************************************/
const DEFAULT_BRANCH = "extensions.netscape.fas.";
const DEFAULT_BUNDLE = "chrome://browser/locale/netscape-fas/fas.properties";
const DEFAULT_SEC = 1000;                                    // 1 second
const DEFAULT_MIN = 60*DEFAULT_SEC;                          // 1 minute
const DEFAULT_HOUR = 60*DEFAULT_MIN;                         // 1 hour
const DEFAULT_DAY = 24*DEFAULT_HOUR;                         // 1 day
const DEFAULT_TIMER = 100;                                   // 100 milliseconds
/*******************************************************************************
 * FAS checker values.
 ******************************************************************************/
const FSC_URL = "http://www.netscape.com/api/friendactivity/";
const FSC_TIMER = 15*DEFAULT_SEC;                        // 15 seconds
const FSC_EXPIRES = 5*DEFAULT_MIN;                       // 5 minutes
const FSC_VAR = 10*DEFAULT_SEC;                          // 10 seconds
/*******************************************************************************
 * URL checker values.
 ******************************************************************************/
const URL_TIMER = 5*DEFAULT_TIMER;                        // 500 milliseconds
/*******************************************************************************
 * Alert manager values.
 ******************************************************************************/
const ALM_TITLE = "fas.module.alert.title";
const ALM_TEXT = "fas.module.alert.text.";
const ALM_IMAGE = "chrome://browser/skin/netscape-fas/fas-alert.png";
const ALM_WIN = "navigator:browser";
/*******************************************************************************
 * FAS service values.
 ******************************************************************************/
const FSS_ERROR = "http://www.mozilla.org/newlayout/xml/parsererror.xml";
const FSS_SIGNIN = "chrome://browser/content/netscape-fas/fas-signin.xul";
const FSS_URL = "http://www.netscape.com/member/%1/fas/";
const FSS_DATA_CHANGE = "change";
const FSS_DATA_SIGNIN = "signin";
const FSS_DATA_SIGNOUT = "signout";
/*******************************************************************************
 * Helper object for doing date time conversions.
 *
 * @param     value          A date value as a millisecond string.
 ******************************************************************************/
function DateUtil(value) {
  this.value = value;
}
DateUtil.prototype = {

  _value: "0",
  get value() { return this._value; },
  set value(val) { this._value = val; },
  
  get date() { return new Date(Number(this.value)); },
  set date(val) { this.value = String(val.getTime()); },  
  
  get time() { return this.date.getTime(); },
  set time(val) { this.date = new Date(val); },
  
  get string() { return this.date.toString(); },
  set string(val) { this.date = new Date(val); },

  get current() { return new Date(); },
  
  // date is less then the current date + a variance
  hasExpired: function du_hasExpired(variance) {
    return (this.time < (this.current.getTime() + variance)); 
  },
  
  // reset the data to current date + value
  reset: function du_reset(value) {
    var time = this.current.getTime() + value;
    var newDate = new Date(time);
    this.date = newDate;
  }
};
/*******************************************************************************
 * Object used to check the friends activity.  This will callback to the
 * fas service.
 *
 * @param     service        The fas service.
 ******************************************************************************/
function FASChecker(service) {
  this.fasService = service;
}
FASChecker.prototype = {

  _timer: null,
  get timer() { return this._timer; },
  set timer(val) { this._timer = val; },

  _firstTime: null,
  get firstTime() { return this._firstTime; },
  set firstTime(val) { this._firstTime = val; },
  
  _request: null,
  get request() { return this._request; },
  set request(val) { this._request = val; },

  _locked: null,
  get locked() { return this._locked; },
  set locked(val) { this._locked = val; },
  
  _expires: null,
  get expires() { return this._expires; },
  set expires(val) { this._expires = val; },
  
  _fasService: null,
  get fasService() { return this._fasService; },
  set fasService(val) { this._fasService = val; },
  
  _isPaused: null,
  get isPaused() { return this._isPaused; },
  set isPaused(val) { this._isPaused = val; },
    
  // nsISupports
  QueryInterface: function fsc_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
        
  // nsIObserver
  observe: function fsc_observe(subject, topic, data) { 
    if (topic == TOPIC_TIMER)
      this.fireTimer();
    else if (topic == TOPIC_PREF)
      this.reload();
  },
   
  needsUpdated: function fsc_needsUpdated(force) {
  
    // activity is being update we do not need another
    if (this.locked)
      return false;
    
    // force so we update
    if (force)
      return true;
      
    // feeds expired we need an update
    var util = new DateUtil(this.expires);
    if (util.hasExpired(FSC_VAR))
      return true;
      
    // no update needed
    return false;    
  },
   
  load: function fsc_load() {

    // add an observer to the pref branch
    this.fasService._branch.QueryInterface(Ci.nsIPrefBranch2).addObserver("", this, false);
    
    // start timer      
    this.expires = "0";
    this.isPaused = false;             
    this.firstTime = true;
    this.timer = Cc[TIMER_CONTRACTID].createInstance(Ci.nsITimer);
    this.timer.init(this, DEFAULT_TIMER, Ci.nsITimer.TYPE_ONE_SHOT);
  },

  unload: function fsc_unload() {

    // remove the pref observer
    this.fasService._branch.QueryInterface(Ci.nsIPrefBranch2).removeObserver("", this);
    
    // stop any request
    if (this.request)
      this.request.abort();
    
    // stop timer  
    this.timer.cancel();
    
    // remove variables
    this.timer = null;    
    this.firstTime = null;    
    this.request = null;  
    this.locked = null;    
    this.expires = null;      
    this.fasService = null;
    this.isPaused = null;
  },
  
  reload: function fsc_reload() {
    // stop request
    if (this.request)
      this.request.abort();
    this.locked = false;
    
    // request again
    this.requestUpdate(true);
  },
  
  pause: function fsc_pause() {
    this.timer.cancel();  
    this.isPaused = true;
  },
  
  resume: function fsc_resume() {
    if (!this.isPaused)
      return;
    this.timer.init(this, FSC_TIMER, Ci.nsITimer.TYPE_REPEATING_SLACK);  
    this.isPaused = false;
  },
  
  fireTimer: function fsc_fireTimer() {

    // change timer to a repeating timer if this is the first run
    if (this.firstTime) {
      this.firstTime = false;
      this.timer.init(this, FSC_TIMER, Ci.nsITimer.TYPE_REPEATING_SLACK);
    }
    
    // request an update
    this.requestUpdate(false);
  },
      
  requestUpdate: function fsc_requestUpdate(force) {
      
    // api does not need updating
    if (!this.needsUpdated(force))
      return;
           
    // create an XML HTTP request  
    var self = this;
    var url = this.fasService._buildURL(FSC_URL);
    var request = Cc[REQUEST_CONTRACTID].createInstance(Ci.nsIXMLHttpRequest);
    request.open("GET", url, true);
    request.overrideMimeType("text/xml");
    request.onerror = function(event) { self.onError(event);    };
    request.onload = function(event) { self.onLoad(event);     };
    this.request = request;
    
    // send the request
    this.locked = true;
    this.request.send(null);
  },

  onLoad: function fsc_onLoad(event) {
  
    // callback the service
    var success = false;
    try {
      success = this.fasService._callback(this.request);  
    } catch(e) {
      success = false;
      Components.utils.reportError(e);
    }
    
    // release the request
    this.locked = false;
    this.request = null;
    
    // set the expires date
    var util = new DateUtil("0");
    var value = FSC_EXPIRES;
    if (success)
      value = this.fasService._branch.getIntPref("frequency")*DEFAULT_MIN;
    util.reset(value);
    this.expires = util.value;
  },

  onError: function fsc_onError(event) {   
    // release the request 
    this.locked = false;
    this.request = null;
    
    // set the expires date
    var util = new DateUtil("0");
    util.reset(FSC_EXPIRES);
    this.expires = util.value;
    
    // make sure we are not paused  
    this.resume();  
  }
};
/*******************************************************************************
 * Object used to check URLs for signin/signout.  This will callback to the
 * fas service.
 *
 * @param     service        The fas service.
 ******************************************************************************/
function URLChecker(service) {
  this.fasService = service;
}
URLChecker.prototype = {

  _timer: null,
  get timer() { return this._timer; },
  set timer(val) { this._timer = val; },

  _firstTime: null,
  get firstTime() { return this._firstTime; },
  set firstTime(val) { this._firstTime = val; },
  
  _fasService: null,
  get fasService() { return this._fasService; },
  set fasService(val) { this._fasService = val; },
  
  _queue: null,
  get queue() { return this._queue; },
  set queue(val) { this._queue = val; },
 
  _homePage: null,
  get homePage() { return this._homePage; },
  set homePage(val) { this._homePage = val; },
  
  _signinPage: null,
  get signinPage() { return this._signinPage; },
  set signinPage(val) { this._signinPage = val; },
  
  _signoutPage: null,
  get signoutPage() { return this._signoutPage; },
  set signoutPage(val) { this._signoutPage = val; },
  
  _activityPage: null,
  get activityPage() { return this._activityPage; },
  set activityPage(val) { this._activityPage = val; },
    
  // nsISupports
  QueryInterface: function url_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
        
  // nsIObserver
  observe: function url_observe(subject, topic, data) { 
    if (topic != TOPIC_TIMER)
      return;
    this.fireTimer();
  },
   
  load: function url_load() {
    
    // setup the regexp
    var pattern = "^https?:\/\/www\.netscape\.com\/signin";    
    this.signinPage = new RegExp(pattern, "i");
    pattern = "^https?:\/\/www\.netscape\.com\/signout";
    this.signoutPage = new RegExp(pattern, "i");
    pattern = "^https?:\/\/www\.netscape\.com$"; 
    this.homePage = new RegExp(pattern, "i");  
    pattern = "^https?:\/\/www\.netscape\.com\/member/[^/]+/friends/recentactivity";  
    this.activityPage = new RegExp(pattern, "i");
    
    // setup the timer     
    this.queue = [];
    this.firstTime = true;
    this.timer = Cc[TIMER_CONTRACTID].createInstance(Ci.nsITimer);
    this.timer.init(this, DEFAULT_TIMER, Ci.nsITimer.TYPE_ONE_SHOT);
  },

  unload: function url_unload() {
  
    // stop the timer
    this.timer.cancel();
    
    // remove variables
    this.timer = null;    
    this.firstTime = null;    
    this.fasService = null;
    this.queue = null;
    this.homePage = null;
    this.signinPage = null;
    this.signoutPage = null;
    this.activityPage = null;
  },

  
  fireTimer: function url_fireTimer() {

    // change timer to a repeating timer if this is the first run
    if (this.firstTime) {
      this.firstTime = false;
      this.timer.init(this, URL_TIMER, Ci.nsITimer.TYPE_REPEATING_SLACK);
    }
    
    // read through the queue
    this.checkQueue();
  },
      
  checkQueue: function url_checkQueue() {
    while (this.queue.length) {
      var url = this.queue.shift();
      this.checkURL(url);    
    }
  },

  addURL: function url_addURL(url) {
    this.queue.push(url);
  },
  
  checkURL: function url_checkURL(url) {   

    // user is signing out
    if (url.match(this.signoutPage))
      this.fasService._clearData(true);
      
    // user is signing in
    else if (url.match(this.signinPage))
      this.fasService.checkActivity();
  
    // user is viewing activity
    else if (url.match(this.activityPage))
      this.fasService.checkActivity();
      
    // user is hitting the homepage
    else if (url.match(this.homePage))
      this.fasService.checkActivity();
  } 
};
/*******************************************************************************
 * Object used to display an alert notification.
 *
 * @param     service        The fas service.
 ******************************************************************************/
function AlertManager(service) {
  this.fasService = service;
}
AlertManager.prototype = {

  _timer: null,
  get timer() { return this._timer; },
  set timer(val) { this._timer = val; },

  _fasService: null,
  get fasService() { return this._fasService; },
  set fasService(val) { this._fasService = val; },
  
  _bundle: null,
  get bundle() { return this._bundle; },
  set bundle(val) { this._bundle = val; },

  _totalCount: null,
  get totalCount() { return this._totalCount; },
  set totalCount(val) { this._totalCount = val; },
  
  _prevCount: null,
  get prevCount() { return this._prevCount; },
  set prevCount(val) { this._prevCount = val; },
  
  // nsISupports
  QueryInterface: function alm_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
        
  // nsIObserver
  observe: function alm_observe(subject, topic, data) { 
    if (topic == TOPIC_ALERT)
      this.open(subject.QueryInterface(Ci.nsIPropertyBag));
    else if (topic == TOPIC_TIMER)
      this.fireTimer();
  },
   
  load: function alm_load() {
    // setup count
    this.totalCount = 0;
    this.prevCount = 0;
    
    // setup string bundle
    var sbService = Cc[BUNDLE_CONTRACTID].getService(Ci.nsIStringBundleService);
    this.bundle = sbService.createBundle(DEFAULT_BUNDLE); 
    
    // setup the timer
    this.timer = Cc[TIMER_CONTRACTID].createInstance(Ci.nsITimer);   
  },

  unload: function alm_unload() {

    // stop the timer
    this.timer.cancel();
    
    // remove the variables
    this.timer = null;
    this.fasService = null;
    this.bundle = null;
    this.totalCount = null;
    this.prevCount = null;
  },

  
  fireTimer: function alm_fireTimer() {
    
    // no alerts
    if (this.totalCount < 1)
      return;
    
    // count has not changed
    if (this.totalCount == this.prevCount)
      return;
          
    // do nothing if we do not have the alert service
    var showAlert = (ALERT_CONTRACTID in Cc);
    if (!showAlert)
      return;
      
    // do nothing if not enabled
    showAlert = this.fasService._branch.getBoolPref("showAlert");
    if (!showAlert)
      return;
    
    // show the alert
    var service = Cc[ALERT_CONTRACTID].getService(Ci.nsIAlertsService);
    var title = this.bundle.GetStringFromName(ALM_TITLE);
    var text;
    if (this.totalCount == 1)
      text = ALM_TEXT + "single";   
    else
      text = ALM_TEXT + "plural";
    text = this.bundle.formatStringFromName(text, [this.totalCount], 1);
    service.showAlertNotification(ALM_IMAGE, title, text, true, null, this);
  },

  showAlert: function alm_showAlert(count) {
    this.prevCount = this.totalCount;
    this.totalCount = count;
    this.timer.init(this, DEFAULT_TIMER, Ci.nsITimer.TYPE_ONE_SHOT);
  },
  
  open: function alm_open(bag) {
    var mediator = Cc[MEDIATOR_CONTRACTID].getService(Ci.nsIWindowMediator);
    var win = mediator.getMostRecentWindow(ALM_WIN);
    if (!win)
      return;
    var e = {
      shiftKey: bag.getProperty("shiftKey"),
      ctrlKey: bag.getProperty("ctrlKey"),
      metaKey: bag.getProperty("metaKey"),
      altKey: bag.getProperty("altKey"),
      button: bag.getProperty("button")    
    };      
    win.FAS.open(e);  
  }
};
/*******************************************************************************
 * The main fas interface.  This interface takes care of updating the 
 * friends' activity list.  It notifies the front-end when updates take place.
 * 
 * @version   1.0
 ******************************************************************************/
function FASService() {}
FASService.prototype = {
  _userName: null,
  _totalCount: null,
  _counts: null,
  _URL: null,
  _obService: null,
  _fasChecker: null,
  _urlChecker: null,
  _alertManager: null,
  _branch: null,
        
  // nsISupports
  QueryInterface: function fas_QI(iid) {
    if (iid.equals(Ci.nsIFASService) ||
        iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  // nsIObserver
  observe: function fas_observe(subject, topic, data) {
    switch (topic) {
          
    case TOPIC_STARTUP_APP:
      this._load();
      break;
    
    case TOPIC_STARTUP_UI:
      this._loadDelayed();
      break;
    
    case TOPIC_SHUTDOWN_APP:
      this._unload();
      break;
              
    case TOPIC_SHUTDOWN_XPCOM:
      this._unloadDelayed();
      break;
    
    case TOPIC_REQUEST:
      var url = subject.QueryInterface(Ci.nsIHttpChannel).originalURI.spec;    
      this._urlChecker.addURL(url);
      break;
      
    default:
      return;
    }
  },
    
  get isSignedIn() { return (this._userName !== null); },
  get userName() { return this._userName; },
  get hasActivity() { return (this._totalCount > 0); },
  get totalCount() { return this._totalCount; },
  get URL() { return this._URL; },
  

  getCount: function fas_getCount(activity) {
    if (!this._counts.hasOwnProperty(activity))
      return 0;
    return this._counts[activity];
  },
  

  checkActivity: function fas_checkActivity() {
    this._fasChecker.requestUpdate(true);
  },
        
  _load: function fas__load() {
    
    // get the pref branch
    this._branch = Cc[PREF_CONTRACTID].getService(Ci.nsIPrefService).
                   getBranch(DEFAULT_BRANCH);
    
    // get the observer service and add ourself as an observer               
    this._obService = Cc[OB_CONTRACTID].getService(Ci.nsIObserverService);
    this._obService.addObserver(this, TOPIC_STARTUP_UI, false);
    this._obService.addObserver(this, TOPIC_SHUTDOWN_APP, false);
    this._obService.addObserver(this, TOPIC_SHUTDOWN_XPCOM, false);
    this._obService.addObserver(this, TOPIC_REQUEST, false);

    // setup the variables
    this._clearData(false);
    this._fasChecker = new FASChecker(this);    
    this._urlChecker = new URLChecker(this);
    this._alertManager = new AlertManager(this);
  },
  
  _loadDelayed: function fas__loadDelayed() {
    this._fasChecker.load();
    this._urlChecker.load();
    this._alertManager.load();
  },
  
  _unload: function fas__unload() {
    this._fasChecker.unload();
    this._urlChecker.unload();
    this._alertManager.unload();    
  },
  
  _unloadDelayed: function fas__unloadDelayed() {
         
    // remove observers
    this._obService.removeObserver(this, TOPIC_STARTUP_UI);
    this._obService.removeObserver(this, TOPIC_SHUTDOWN_APP);    
    this._obService.removeObserver(this, TOPIC_SHUTDOWN_XPCOM);
    this._obService.removeObserver(this, TOPIC_REQUEST);
    
    // remove variables
    this._userName = null;
    this._totalCount = null;
    this._counts = null;
    this._URL = null;
    this._obService = null;    
    this._fasChecker = null;
    this._urlChecker = null;
    this._alertManager = null;
    this._branch = null;   
  },
  
  _callback: function fas__callback(request) {
    var wasSignedIn = this.isSignedIn;

    // on request error unpause and return error
    var xml = request.responseXML;
    if (!xml || xml.documentElement.namespaceURI == FSS_ERROR ||
       (request.status != 200 && request.status !== 0)) {
      this._fasChecker.resume();
      return false;
    }  
    
    // reset the values and return if signed out
	var member = xml.getElementsByTagName("member")[0];
	var signedIn = parseInt(member.getAttribute("loggedIn"));
    this._clearData(!signedIn);
	if (!signedIn)
      return true;

    // resume the timer
    this._fasChecker.resume();

    // setup the sidebar url
    this._userName = member.getElementsByTagName("alias")[0].firstChild.nodeValue;    
    var url = FSS_URL.replace("%1", this.userName);
    this._URL = this._buildURL(url);

    // setup the activity counts
    var names = { "stories": "storycount", "votes": "votecount", "comments": "commentcount" };
    for (var name in names) {
      if (this._branch.getBoolPref("notifications." + name))
        this._counts[name] = parseInt(xml.getElementsByTagName(name)[0].getElementsByTagName(names[name])[0].firstChild.nodeValue);
      this._totalCount += this._counts[name];
    }
    this._alertManager.showAlert(this._totalCount);
    
    // notify the front-end we changed and return success
    data = (!wasSignedIn) ? FSS_DATA_SIGNIN : FSS_DATA_CHANGE;
    this._notify(data);
    return true;
  },
  
  _buildURL: function fas__buildURL(url) {
  
    // loop through possible options
    var delimiter = "?";  
    var names = ["stories", "comments", "votes"];
    for (var i=0; i<names.length; i++) {
    
      // skip if not active
      if (!this._branch.getBoolPref("notifications." + names[i]))
        continue;
        
      // add to url and switch delimiter
      url += delimiter + names[i] + "=1";
      delimiter = "&";
    }
    
    // return the new url
    return url;
  },
  
  _clearData: function fas__clearData(notify) {
    var wasSignedIn = this.isSignedIn;
    
    this._userName = null;
    this._totalCount = 0;
    this._counts = { "stories": 0, "votes": 0, "comments": 0 };
    this._URL = FSS_SIGNIN;  
  
    if (notify) {
      this._fasChecker.pause();
      var data = (wasSignedIn) ? FSS_DATA_SIGNOUT : FSS_DATA_CHANGE;
      this._notify(data);
    }
  },
  
  _notify: function fas__notify(data) {
    try {
      this._obService.notifyObservers(this, TOPIC_FAS, data);
    } catch(e) {
      Components.utils.reportError(e);
    }
  }    
};
/*******************************************************************************
 * Generic object used as a factory based on a passed in constructor.
 ******************************************************************************/
function GenericFactory(ctor) {
  this._ctor = ctor;
}  
GenericFactory.prototype = {
  _ctor: null,
  
  // nsISupports
  QueryInterface: function gf_QI(iid) {
    if (iid.equals(Ci.nsIFactory) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  // nsIFactory
  createInstance: function gf_createInstance(outer, iid) {
    if (outer !== null)
      throw Cr.NS_ERROR_NO_AGGREGATION;
    return (new this._ctor()).QueryInterface(iid);
  },

  lockFactory: function gf_lockFactory(loc) {
    throw Cr.NS_ERROR_NOT_IMPLEMENTED;
  }
};
/*******************************************************************************
 * Object implementing the nsIModule interface.  This is used to retrieve 
 * the different components and services the module implements.
 ******************************************************************************/
var Module = {

  // nsISupports
  QueryInterface: function mod_QI(iid) {
    if (iid.equals(Ci.nsIModule) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE;
  },
  
  // nsIModule
  getClassObject: function mod_getClassObject(cm, cid, iid) {
    if (!iid.equals(Ci.nsIFactory))
      throw Cr.NS_ERROR_NOT_IMPLEMENTED;
    
    if (cid.equals(FSS_CLASSID))
      return new GenericFactory(FASService);
          
    throw Cr.NS_ERROR_NO_INTERFACE;
  },

  registerSelf: function mod_registerSelf(cm, file, location, type) {
    var cr = cm.QueryInterface(Ci.nsIComponentRegistrar);  
    cr.registerFactoryLocation(FSS_CLASSID, FSS_CLASSNAME, FSS_CONTRACTID,
                               file, location, type);   
                                
    var catMgr = Components.classes[CAT_CONTRACTID].
                 getService(Ci.nsICategoryManager);
    catMgr.addCategoryEntry(TOPIC_STARTUP_APP, FSS_CLASSNAME, "service," + 
                            FSS_CONTRACTID, true, true);                                                                                         
  },

  unregisterSelf: function mod_unregisterSelf(cm, location, type) {
    var cr = cm.QueryInterface(Ci.nsIComponentRegistrar);
    cr.unregisterFactoryLocation(FSS_CLASSID, location);
                                
    var catMgr = Components.classes[CAT_CONTRACTID].
                 getService(Ci.nsICategoryManager); 
    catMgr.deleteCategoryEntry(TOPIC_STARTUP_APP, "service," + 
                               FSS_CONTRACTID, true);
  },

  canUnload: function mod_canUnload(cm) { return true; }
};
/*******************************************************************************
 * Entry point used to retrieve the module object.
 ******************************************************************************/
function NSGetModule(cm, file) { return Module; }